<?php

namespace Hrm\Customer\Providers;

use Illuminate\Support\ServiceProvider;

class CustomerServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'customer');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'customer');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerCustomer();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerCustomer()
    {
        $this->app->bind('customer', function($app) {
            return new Customer($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Customer', 'Lavalite\Customer\Facades\Customer');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('hrm.customer', function ($app) {
            return $this->app->make('Hrm\Customer\Customer');
        });

        // Bind Customer to repository
        $this->app->bind(
            'Hrm\Customer\Interfaces\CustomerRepositoryInterface',
            \Hrm\Customer\Repositories\Eloquent\CustomerRepository::class
        );
        // Bind Call to repository
        $this->app->bind(
            'Hrm\Customer\Interfaces\CallRepositoryInterface',
            \Hrm\Customer\Repositories\Eloquent\CallRepository::class
        );
        // Bind Status to repository
        $this->app->bind(
            'Hrm\Customer\Interfaces\StatusRepositoryInterface',
            \Hrm\Customer\Repositories\Eloquent\StatusRepository::class
        );

        $this->app->register(\Hrm\Customer\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Hrm\Customer\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and customer's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'hrm.customer'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeCustomer::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['hrm.customer'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('hrm/customer.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/customer')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/customer')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
